#include "ComponentTraverseInfo.h"

namespace AMDiS {
  const int SingleComponentInfo::EMPTY = 0;
  const int SingleComponentInfo::EQ_SPACES_NO_AUX = 1;
  const int SingleComponentInfo::EQ_SPACES_WITH_AUX = 2;
  const int SingleComponentInfo::EQ_SPACES_WITH_DIF_AUX = 3;
  const int SingleComponentInfo::DIF_SPACES_NO_AUX = 4;
  const int SingleComponentInfo::DIF_SPACES_WITH_AUX = 5;
  const int SingleComponentInfo::DIF_SPACES_WITH_DIF_AUX = 6;

  void SingleComponentInfo::updateStatus()
  {
    if (rowFeSpace == NULL) {
      status = SingleComponentInfo::EMPTY;
      return;
    }

    if (colFeSpace == NULL || 
	(colFeSpace != NULL && rowFeSpace->getMesh() == colFeSpace->getMesh())) {
      if (auxFeSpaces.size() == 0) {
	status = SingleComponentInfo::EQ_SPACES_NO_AUX;
      } else {
	status = SingleComponentInfo::EQ_SPACES_WITH_AUX;
	for (int i = 0; i < static_cast<int>(auxFeSpaces.size()); i++) {
	  if (auxFeSpaces[i]->getMesh() != rowFeSpace->getMesh()) {
	    status = SingleComponentInfo::EQ_SPACES_WITH_DIF_AUX;
	    break;
	  }
	}
      }
    } else {
      if (auxFeSpaces.size() == 0) {
	status = SingleComponentInfo::DIF_SPACES_NO_AUX;
      } else {
	status = SingleComponentInfo::DIF_SPACES_WITH_AUX;
	for (int i = 0; i < static_cast<int>(auxFeSpaces.size()); i++) {
	  if (auxFeSpaces[i]->getMesh() != rowFeSpace->getMesh() &&
	      auxFeSpaces[i]->getMesh() != colFeSpace->getMesh()) {
	    status = SingleComponentInfo::DIF_SPACES_WITH_DIF_AUX;
	    break;
	  }
	}
      }	
    }    
  }

  
  const FiniteElemSpace* ComponentTraverseInfo::getRowFeSpace(int row)
  {
    FUNCNAME("ComponentTraverseInfo::getRowFeSpace()");
    
    TEST_EXIT_DBG(row < nComponents)("No component traverse info for this row!\n");
    TEST_EXIT_DBG(matrixComponents[row][row].getRowFeSpace() ==
		  matrixComponents[row][row].getColFeSpace())
      ("Should not happen!\n");
    
    return matrixComponents[row][row].getRowFeSpace();      
  }


  const FiniteElemSpace* ComponentTraverseInfo::getNonRowFeSpace(int row)
  {
    FUNCNAME("ComponentTraverseInfo::getNonRowFeSpace()");

    TEST_EXIT_DBG(row < nComponents)("No component traverse info for this row!\n");
    
    const FiniteElemSpace* rowFeSpace = getRowFeSpace(row);
    
    TEST_EXIT_DBG(rowFeSpace != NULL)("No row FE space!\n");
    
    for (int i = 0; i < nComponents; i++) {
      if (matrixComponents[row][i].getColFeSpace() != rowFeSpace)
	return matrixComponents[row][i].getColFeSpace();
      if (matrixComponents[row][i].getAuxFeSpace(0) != rowFeSpace)
	return matrixComponents[row][i].getAuxFeSpace(0);
    }
    
    if (vectorComponents[row].getAuxFeSpace(0) != rowFeSpace)
      return vectorComponents[row].getAuxFeSpace(0);
    
    return NULL;
  }

}
